#!/usr/bin/python
import os, datetime, sys, re
from os import path

com_re = re.compile(r"^\s*#.*$")
set_re = re.compile(r'^\s*([^=\s]+)\s*=\s*(")?(.*)\2\s*$')
cfgfile = file("test-config","r")
cfgs = { }
for l in cfgfile.readlines():
    if com_re.match(l):
        continue
    m = set_re.match(l)
    if m != None:
        cfgs[m.group(1)] = m.group(3)

UML_EXE = path.expandvars(path.expanduser(cfgs['UML_EXE']))
UML_FS = path.expandvars(path.expanduser(cfgs['UML_FS']))
INITNG_SRC = path.expandvars(path.expanduser(cfgs['INITNG_SRC']))
OUT_DIR = path.expandvars(path.expanduser(cfgs['OUT_DIR']))
bl_re = re.compile(r"^initng-builder:\s+(.*?)\s*$")
tl_re = re.compile(r"^initng-tester:\s+(.*?)\s*$")
res_re = re.compile(r"^(K?PASS|K?FAIL|SKIP) (.*)$")
ansi_esc_re = re.compile(r"\x1b\[[\d;]+m")

####################
# xterm titles support shamelessly ripped from Portage
# Copyright 1998-2004 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

def xtermTitle(mystr):
    if os.environ.has_key("TERM") and sys.stderr.isatty():
        myt=os.environ["TERM"]
        legal_terms = ["xterm","Eterm","aterm","rxvt","screen","kterm","rxvt-unicode"]
        for term in legal_terms:
            if myt.startswith(term):
                sys.stderr.write("\x1b]2;"+str(mystr)+"\x07")
                sys.stderr.flush()
                break

def xtermTitleReset():
    if os.environ.has_key("TERM"):
        xtermTitle(os.environ["TERM"])

####################


def do_log(s,*files):
    sys.stdout.write(s)
    for f in files:
        f.write(s)

def make_sys():
    outdir = OUT_DIR + "/" + datetime.datetime.utcnow().isoformat('_')
    os.mkdir(outdir)
    logfile = file(outdir + "/build.log","w",0)
    do_log("run-test.py: running build to " + outdir + "\n",logfile)
    xtermTitle("initng-build: booting")
    cmd = (UML_EXE + " ubd0=" + outdir + "/cow.out," + UML_FS +
           " init=/sbin/initng mem=64M runlevel:update")
    pipes = os.popen4(cmd,None,0); c_stdin = pipes[0]; c_allout=pipes[1]
    last_status = ""
    while True:
        d = c_allout.readline()
        if d == '': break
        m = bl_re.match(d)
        do_log(d,logfile)
        if m != None:
            last_status=m.group(1)
            xtermTitle("initng-build: " + last_status)
            do_log("run_test.py: got status '" + last_status + "'\n",logfile)
    if last_status == "OK":
        do_log("run_test.py: built OK to " + outdir + "\n",logfile)
        logfile.close(); xtermTitleReset(); return outdir
    else:
        do_log("run_test.py: build failed\n",logfile)
        logfile.close(); xtermTitleReset(); return None

def do_tests(outdir):
    stats = {'PASS': 0, 'KPASS': 0, 'FAIL': 0, 'KFAIL': 0, 'SKIP': 0 }
    tcount = 0
    logfile = file(outdir + "/tests.log","w",0)
    reslog = file(outdir + "/results","w",0)
    do_log("run-test.py: running tests on " + outdir + "\n",logfile)
    cmd = (UML_EXE + " ubd0=" + outdir + "/cow.out," + UML_FS +
           " init=/sbin/initng mem=64M runlevel:runtests")
    pipes = os.popen4(cmd,None,0); c_stdin = pipes[0]; c_allout=pipes[1]
    xtermTitle("initng-test: booting")
    last_status = ""; 
    while True:
        d = c_allout.readline()
	if d == '': break
        do_log(d,logfile)
        d = ansi_esc_re.sub("",d)
	m = tl_re.match(d)
	if m != None:
            last_status = m.group(1)
            reslog.write(last_status+"\n")
            m = res_re.match(last_status)
            if m != None:
                stats[m.group(1)] += 1; tcount += 1
                xtermTitle("initng-test: %iPA %iFA %iKP %iKF %iSK" %
                           (stats['PASS'], stats['FAIL'],
                            stats['KPASS'], stats['KFAIL'], stats['SKIP']))
    if tcount == 0:
        do_log("initng-test: FAILED TO RUN TESTS!\n",logfile,reslog)
    else:
            do_log("*** initng-tester SUMMARY ***\n",logfile,reslog)
            for s in ('PASS','FAIL','KPASS','KFAIL','SKIP'):
                do_log((s+":").ljust(8) + " "  + str(stats[s])+"\n",
                       logfile, reslog)
            if(last_status != "Finished!"):
                do_log("FAILED TO COMPLETE TESTS\n",logfile,reslog)
            do_log("*****************************\n",logfile,reslog)
    xtermTitleReset()


def _main():
    if len(sys.argv) > 2:
        sys.exit(1)
    elif len(sys.argv) == 2:
        do_tests(sys.argv[1])
    else:
        outdir = make_sys()
        if outdir == None:
            return
        do_tests(outdir)

if __name__ == '__main__':
    _main()
